/*****************************************************************************
* Lien Mp3 Player is an audio CD and mp3 player for GNU/Linux                *
* Copyright (C) 2007 Tan Chee Kien. See file COPYING for licence details     *
*                                                                            *
*    This program is free software; you can redistribute it and/or modify    *
*    it under the terms of the GNU General Public License as published by    *
*    the Free Software Foundation; either version 2 of the License, or       *
*    any later version.                                                      *
*                                                                            *
*    This program is distributed in the hope that it will be useful,         *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*    GNU General Public License for more details.                            *
*                                                                            *
*    You should have received a copy of the GNU General Public License       *
*    along with this program; if not, write to the Free Software             *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    *  
*    USA                                                                     *
*                                                                            *
*    You can contact the author by sending an email to :                     *
*       gogota321@yahoo.com                                                  *
*****************************************************************************/

/* file selection menu */

#define _GNU_SOURCE
#include "filemenu.h"
#include "lmp3.h"
#include "audiocd.h"

static DIRINFO cwdinfo;
extern LMP3GEN *playopt;

void
filemenu (void)
{
  int totalfile;
  int totaldir;
  int quit;
  int mode;

  // start_ncurses();
  quit = NO;
  mode = MP3MODE;
  while (quit != YES)
    {
      cwdinfo.total_item = 0;
      if (mode == MP3MODE)
	{
	  totalfile = check_total_file ();
	  totaldir = check_total_dir ();
	  make_dir_array (totaldir);
	  make_file_array (totalfile);
	  copy_to_modified (cwdinfo.head);
	  //print_list(cwdinfo.head);
	}
      else			//audio cd mode

	{
	  totalfile = howmanyacdtrack (playopt->cd_device);
	  totaldir = 0;
	}
      display (totalfile + totaldir, &quit, &mode);
      if (mode == MP3MODE)
	clear_list (cwdinfo.head);
    }

}



void
make_file_array (const int totalfile)
{
  DIR *dir;
  char dirname[MAX_LEN];
  struct dirent *files;
  char *file;
  char *temp;
  char *filename[totalfile];
  int count;
  struct stat abc;

  getcwd (dirname, sizeof (dirname));
  dir = opendir (dirname);

  for (count = 0; count < totalfile;)
    {
      files = readdir (dir);
      file = (files->d_name);
      stat (file, &abc);
      if (S_ISDIR (abc.st_mode) != 1)	//it is a file

	{
	  if (fnmatch ("*.*3", file, 0) == 0)
	    {
	      temp = (char *) malloc (sizeof (char) * MAX_LEN);
	      memcheck (temp);
	      memset (temp, 0, MAX_LEN);
	      strncpy (temp, file, MAX_LEN - 1);
	      filename[count] = temp;
	      count++;
	    }
	}
    }

  closedir (dir);
  qsort (filename, totalfile, sizeof (char *), sortme);

  for (count = 0; count < totalfile; count++)
    {
      append_to_list (filename[count], ISFILE);
      free (filename[count]);
      filename[count] = NULL;
    }

}

void
make_dir_array (const int totaldir)
{
  DIR *dir;
  char dirname[MAX_LEN];
  struct dirent *files;
  char *file;
  char *temp;
  char *dir_name[totaldir];
  int count;
  struct stat abc;

  getcwd (dirname, sizeof (dirname));
  dir = opendir (dirname);

  for (count = 0; count < totaldir;)
    {
      files = readdir (dir);
      file = (files->d_name);
      stat (file, &abc);
      if (S_ISDIR (abc.st_mode) == 1)	//it is a dir

	{
	  temp = (char *) malloc (sizeof (char) * MAX_LEN);
	  memcheck (temp);
	  strncpy (temp, file, MAX_LEN - 1);
	  // printf("%s\n",temp);
	  dir_name[count] = temp;
	  count++;
	}
    }

  closedir (dir);
  qsort (dir_name, totaldir, sizeof (char *), sortme);

  for (count = 0; count < totaldir; count++)
    {
      append_to_list (dir_name[count], ISDIR);
      free (dir_name[count]);
      dir_name[count] = NULL;
    }
}

int
check_total_dir (void)
{
  DIR *dir;
  struct dirent *files;
  char dirname[MAX_LEN];
  int count = 0;
  struct stat abc;
  char *file;

  getcwd (dirname, sizeof (dirname));
  dir = opendir (dirname);

  while ((files = readdir (dir)) != NULL)
    {
      file = (files->d_name);
      stat (file, &abc);
      if (S_ISDIR (abc.st_mode) == 1)	//it is a dir

	count++;

    }
  closedir (dir);

  return count;
}

int
check_total_file (void)
{
  DIR *dir;
  struct dirent *files;
  char *dirname;
  int count = 0;
  struct stat abc;
  char *file;

  dirname = get_current_dir_name ();

  //erase();
  //printw("%s",dirname);
  //refresh();
  //getch();

  dir = opendir (dirname);

  while (1)
    {
      files = readdir (dir);
      if (files == 0)
	break;
      else
	{
	  file = (files->d_name);


	  stat (file, &abc);
	  if (S_ISDIR (abc.st_mode) != 1)	//it is a file

	    {
	      if (fnmatch ("*.*3", file, 0) == 0)
		count++;
	    }
	}
    }
  closedir (dir);

  return count;
}

int
sortme (const void *first, const void *second)
{
  return strcmp (*(char **) first, *(char **) second);
}

void
append_to_list (const char *string, const int DirOrFile)
{
  cwdinfo.total_item++;
  if (cwdinfo.total_item == 1)	//1st item, hehe

    {
      cwdinfo.head = (LIST *) malloc (sizeof (LIST));
      memcheck (cwdinfo.head);
      strncpy ((cwdinfo.head)->filename, string, MAX_LEN - 1);
      (cwdinfo.head)->dir_or_file = DirOrFile;
      (cwdinfo.head)->item_num = cwdinfo.total_item;
      (cwdinfo.head)->next = NULL;
      cwdinfo.tail = cwdinfo.head;
    }
  else
    {
      (cwdinfo.tail)->next = (LIST *) malloc (sizeof (LIST));
      memcheck (cwdinfo.tail);
      cwdinfo.tail = (cwdinfo.tail)->next;
      strncpy ((cwdinfo.tail)->filename, string, MAX_LEN - 1);
      (cwdinfo.tail)->dir_or_file = DirOrFile;
      (cwdinfo.tail)->item_num = cwdinfo.total_item;
      (cwdinfo.tail)->next = NULL;
    }
}

void
print_list (LIST * a)
{
  printf ("%s, %d\n", a->modified, a->dir_or_file);
  if (a->next != NULL)
    print_list (a->next);
}

void
copy_to_modified (LIST * a)
{
  if (a->dir_or_file == ISDIR)
    {
      strncpy (a->modified, a->filename, MAX_CHAR);
      strcat (a->modified, "/");
    }
  else
    strncpy (a->modified, a->filename, MAX_CHAR);
  if (a->next != NULL)
    copy_to_modified (a->next);
}

void
clear_list (LIST * a)
{
  if (a->next != NULL)
    clear_list (a->next);
  free (a);
  a = NULL;

}

void
display (const int totalfile, int *const quit, int *const mode)
{
#define ENTER 13
#define HOME 262
#define END 360
#define YES 1
#define NO 0
#define SPACE 32
#define F10 274
#define F5 269
#define F4 268
#define F3 267
#define F2 266
#define F1 265


// refer to lmp3func.c for what the color pair is
#define DIRCOLOR  COLOR_PAIR(27)
#define INSTRCOLOR COLOR_PAIR(20)
#define SELCOLOR  COLOR_PAIR(3)
#define FILECOLOR COLOR_PAIR(14)
#define MENUBACKCOLOR COLOR_PAIR(13)
#define MENUFORECOLOR COLOR_PAIR(17)
#define MENUGREYCOLOR COLOR_PAIR(8)
#define KEYCOLOR COLOR_PAIR(21)

  ITEM *item[totalfile + 1];
  char *filesname[totalfile + 1];
  MENU *filemenu;
  WINDOW *filewin;
  WINDOW *subfilewin;
  WINDOW *dirwin;
  WINDOW *selwin;		//window that display file selected

  WINDOW *instruct, *subinst;	//the window that teach the user how to use

  char temp[MAX_CHAR + 1];
  char temp2[MAX_LEN];
  int count;
  int key;
  char *dirname = NULL;
  int reload;
  int totalcdtrack;
  FILE *playlist;
  char *tmp_file;
  char bsa_dir[300];
  char bsa_file[300];

  wbkgd (stdscr, SELCOLOR);
  erase ();
  refresh ();

  instruct = newwin (24, 10, 0, 0);
  wbkgd (instruct, INSTRCOLOR);
  wattron (instruct, INSTRCOLOR + A_BOLD);
  box (instruct, ACS_VLINE, ACS_HLINE);
  wrefresh (instruct);
  subinst = subwin (instruct, 0, 0, 1, 1);
  wattron (subinst, INSTRCOLOR + A_BOLD);
  mvwprintw (subinst, 0, 0, "F1/p : ");
  wattron (subinst, KEYCOLOR + A_BOLD);
  mvwprintw (subinst, 1, 0, "Playlist");
  mvwprintw (subinst, 2, 0, "Menu");
  wattron (subinst, INSTRCOLOR + A_BOLD);
  mvwprintw (subinst, 3, 0, "F2/o : ");
  wattron (subinst, KEYCOLOR + A_BOLD);
  mvwprintw (subinst, 4, 0, "Options");
  mvwprintw (subinst, 5, 0, "Menu");
  wattron (subinst, INSTRCOLOR + A_BOLD);
  mvwprintw (subinst, 6, 0, "F3/a : ");
  wattron (subinst, KEYCOLOR + A_BOLD);
  mvwprintw (subinst, 7, 0, "Select A");
  mvwprintw (subinst, 8, 0, "ll Files");
  wattron (subinst, INSTRCOLOR + A_BOLD);
  mvwprintw (subinst, 9, 0, "F4/m : ");
  wattron (subinst, KEYCOLOR + A_BOLD);
  mvwprintw (subinst, 10, 0, "mp3/audi");
  mvwprintw (subinst, 11, 0, "o cd");
  wattron (subinst, INSTRCOLOR + A_BOLD);
  mvwprintw (subinst, 12, 0, "F5/b :");
  wattron (subinst, KEYCOLOR + A_BOLD);
  mvwprintw (subinst, 13, 0, "Big Sele");
  mvwprintw (subinst, 14, 0, "ct All");
  wattron (subinst, INSTRCOLOR + A_BOLD);
  mvwprintw (subinst, 15, 0, "F10 : ");
  wattron (subinst, KEYCOLOR + A_BOLD);
  mvwprintw (subinst, 16, 0, "Done And");
  mvwprintw (subinst, 17, 0, "Play !");
  wattron (subinst, INSTRCOLOR + A_BOLD);
  mvwprintw (subinst, 18, 0, "SPACE :");
  wattron (subinst, KEYCOLOR + A_BOLD);
  mvwprintw (subinst, 19, 0, "Select");
  //mvwprintw(subinst,18,0,"Use Arro");
  //mvwprintw(subinst,19,0,"w, PgUp");
  //mvwprintw(subinst,20,0,"& PgDn t");
  //mvwprintw(subinst,21,0,"o Browse");
  //mvwprintw(subinst,22,0,"Menu");
  wnoutrefresh (subinst);
  dirwin = newwin (3, 70, 0, 10);
  wbkgd (dirwin, DIRCOLOR);
  wattron (dirwin, DIRCOLOR + A_BOLD);
  box (dirwin, ACS_VLINE, ACS_HLINE);
  wattron (dirwin, COLOR_PAIR (25) + A_BOLD);


  if (*mode == MP3MODE)
    {
      for (count = 0; count < totalfile; count++)
	{
	  give_modified_name (count + 1, cwdinfo.head, temp);
	  filesname[count] = (char *) malloc (sizeof (char) * (MAX_CHAR + 1));
	  strncpy (filesname[count], temp, MAX_CHAR + 1);
	  item[count] = new_item (filesname[count], "");
	}
      item[count] = NULL;
    }

  else
    {
      totalcdtrack = howmanyacdtrack (playopt->cd_device);
      mvwprintw (dirwin, 1, 1, "Audio CD Mode");
      wrefresh (dirwin);
      for (count = 0; count < totalfile; count++)
	{
	  filesname[count] = (char *) malloc (sizeof (char) * (4));
	  sprintf (filesname[count], "%d", count + 1);
	  item[count] = new_item (filesname[count], "");
	}
      item[count] = NULL;
    }


  filemenu = new_menu (item);
  filewin = newwin (19, 70, 5, 10);
  wbkgd (filewin, FILECOLOR);
  wattrset (filewin, FILECOLOR + A_BOLD);
  box (filewin, ACS_VLINE, ACS_HLINE);
  // wrefresh(filewin);
  subfilewin = subwin (filewin, 0, 0, 6, 11);
  clearok (subfilewin, TRUE);
  leaveok (filewin, TRUE);
  clearok (filewin, TRUE);
  leaveok (subfilewin, TRUE);

  set_menu_win (filemenu, subfilewin);
  set_menu_back (filemenu, MENUBACKCOLOR + A_BOLD);
  set_menu_mark (filemenu, "");
  set_menu_format (filemenu, 17, 1);
  set_menu_fore (filemenu, MENUFORECOLOR + A_BOLD);
  set_menu_grey (filemenu, MENUGREYCOLOR);


  if (*mode == MP3MODE)
    {
      dirname = get_current_dir_name ();
      //wattron(dirwin,YELLOW);
      wmove (dirwin, 1, 1);
      memset (temp2, 0, 100);
      strncpy (temp2, dirname, 67);
      wprintw (dirwin, "%s", temp2);
      wnoutrefresh (dirwin);
    }

  selwin = newwin (0, 70, 3, 10);
  wbkgd (selwin, SELCOLOR);
  //wattron(selwin,COLOR_PAIR(23)+A_BOLD);
  wnoutrefresh (selwin);

  post_menu (filemenu);
  //refresh();
  wnoutrefresh (subfilewin);
  wnoutrefresh (filewin);
  doupdate ();
  *quit = YES;

  key = 0;
  nodelay (stdscr, FALSE);	// Patch by Nils Olav

  while (key != F10)
    {
      key = getch ();
//       move(0,0);
      //       printw("%d\n",key);
      //       refresh(); 
      switch (key)
	{

	case KEY_UP:
	  menu_driver (filemenu, REQ_UP_ITEM);
	  post_menu (filemenu);
	  wnoutrefresh (subfilewin);
	  wnoutrefresh (filewin);
	  doupdate ();
	  break;

	case KEY_DOWN:
	  menu_driver (filemenu, REQ_DOWN_ITEM);
	  post_menu (filemenu);
	  wnoutrefresh (subfilewin);
	  wnoutrefresh (filewin);
	  doupdate ();
	  break;

	case KEY_NPAGE:
	  menu_driver (filemenu, REQ_SCR_DPAGE);
	  post_menu (filemenu);
	  wnoutrefresh (subfilewin);
	  wnoutrefresh (filewin);
	  doupdate ();
	  break;

	case KEY_PPAGE:
	  menu_driver (filemenu, REQ_SCR_UPAGE);
	  post_menu (filemenu);
	  wnoutrefresh (subfilewin);
	  wnoutrefresh (filewin);
	  doupdate ();
	  break;

	case HOME:
	  menu_driver (filemenu, REQ_FIRST_ITEM);
	  post_menu (filemenu);
	  wnoutrefresh (subfilewin);
	  wnoutrefresh (filewin);
	  doupdate ();
	  break;

	case END:
	  menu_driver (filemenu, REQ_LAST_ITEM);
	  post_menu (filemenu);
	  wnoutrefresh (subfilewin);
	  wnoutrefresh (filewin);
	  doupdate ();
	  break;

	case F4:
	case 'm':
	  if (*mode == MP3MODE)
	    {
	      if (howmanyacdtrack (playopt->cd_device) != 0)
		{
		  *mode = ACDMODE;
		  key = F10;
		  *quit = NO;
		}
	      else
		{
		  werase (dirwin);
		  wattron (dirwin, DIRCOLOR + A_BOLD);
		  box (dirwin, ACS_VLINE, ACS_HLINE);
		  wattron (dirwin, COLOR_PAIR (25) + A_BOLD);
		  mvwprintw (dirwin, 1, 1,
			     "error reading cdrom device or no cd");
		  wrefresh (dirwin);
		}
	    }
	  else			//audio cd mode

	    {
	      *mode = MP3MODE;
	      key = F10;
	      *quit = NO;
	    }
	  break;

	case ENTER:
	  if (*mode == MP3MODE)
	    {
	      give_dirname (item_index (current_item (filemenu)) + 1,
			    cwdinfo.head, temp2);
	      if (fnmatch ("*/", item_name (current_item (filemenu)), 0) == 0)
		{
		  if (chdir (temp2) == 0)
		    {
		      key = F10;
		      *quit = NO;
		    }
		}
	    }

	  break;

	case F2:
	case 'o':
	  optmenu ();
	  key = F10;
	  *quit = NO;
	  break;


	case SPACE:
	  if (*mode == MP3MODE)
	    {
	      if (fnmatch ("*/", item_name (current_item (filemenu)), 0) != 0)
		{
		  if (playopt->totalsongs == 0)
		    {
		      playopt->first =
			(PLAYLIST *) malloc (sizeof (PLAYLIST));
		      memcheck (playopt->first);
		      playopt->totalsongs++;
		      playopt->first->songnum = playopt->totalsongs;
		      playopt->first->next = NULL;
		      strncpy (playopt->first->dirname, dirname, MAX_LEN - 1);
		      give_original_name (item_index (current_item (filemenu))
					  + 1, cwdinfo.head,
					  playopt->first->filename);
		      playopt->last = playopt->first;
		    }
		  else
		    {
		      playopt->last->next =
			(PLAYLIST *) malloc (sizeof (PLAYLIST));
		      playopt->last = playopt->last->next;
		      memcheck (playopt->last);
		      playopt->totalsongs++;
		      playopt->last->songnum = playopt->totalsongs;
		      playopt->last->next = NULL;
		      strncpy (playopt->last->dirname, dirname, MAX_LEN - 1);
		      give_original_name (item_index (current_item (filemenu))
					  + 1, cwdinfo.head,
					  playopt->last->filename);
		    }

		  werase (selwin);
		  wmove (selwin, 1, 1);
		  wattron (selwin, SELCOLOR + A_BOLD);
		  wprintw (selwin, "%s", item_name (current_item (filemenu)));
		  wrefresh (selwin);
		  //      item_opts_off(current_item(filemenu),O_ONEVALUE);  
		  item_opts_off (current_item (filemenu), O_SELECTABLE);
		  menu_driver (filemenu, REQ_DOWN_ITEM);
		  post_menu (filemenu);
		  box (filewin, ACS_VLINE, ACS_HLINE);
		  wnoutrefresh (subfilewin);
		  wnoutrefresh (filewin);
		  doupdate ();
		}
	    }
	  else			//audio cd mode

	    {
	      if (playopt->totalsongs == 0)
		{
		  playopt->first = (PLAYLIST *) malloc (sizeof (PLAYLIST));
		  memcheck (playopt->first);
		  playopt->totalsongs++;
		  playopt->first->songnum = playopt->totalsongs;
		  playopt->first->next = NULL;
		  strcpy (playopt->first->dirname, "audio cd");
		  strncpy (playopt->first->filename,
			   item_name (current_item (filemenu)), 5);
		  playopt->last = playopt->first;
		}
	      else
		{
		  playopt->last->next =
		    (PLAYLIST *) malloc (sizeof (PLAYLIST));
		  playopt->last = playopt->last->next;
		  memcheck (playopt->last);
		  playopt->totalsongs++;
		  playopt->last->songnum = playopt->totalsongs;
		  playopt->last->next = NULL;
		  strcpy (playopt->last->dirname, "audio cd");
		  strncpy (playopt->last->filename,
			   item_name (current_item (filemenu)), 5);
		}

	      werase (selwin);
	      wmove (selwin, 1, 1);
	      wattron (selwin, SELCOLOR + A_BOLD);
	      wprintw (selwin, "%s", item_name (current_item (filemenu)));
	      wrefresh (selwin);
	      //      item_opts_off(current_item(filemenu),O_ONEVALUE);  
	      item_opts_off (current_item (filemenu), O_SELECTABLE);
	      menu_driver (filemenu, REQ_DOWN_ITEM);
	      post_menu (filemenu);
	      box (filewin, ACS_VLINE, ACS_HLINE);
	      wnoutrefresh (subfilewin);
	      wnoutrefresh (filewin);
	      doupdate ();

	    }
	  break;

	case F3:
	case 'a':
	  menu_driver (filemenu, REQ_FIRST_ITEM);
	  do
	    {
	      if (*mode == MP3MODE)
		{
		  if (fnmatch ("*/", item_name (current_item (filemenu)), 0)
		      != 0)
		    {
		      if (playopt->totalsongs == 0)
			{
			  playopt->first =
			    (PLAYLIST *) malloc (sizeof (PLAYLIST));
			  memcheck (playopt->first);
			  playopt->totalsongs++;
			  playopt->first->songnum = playopt->totalsongs;
			  playopt->first->next = NULL;
			  strncpy (playopt->first->dirname, dirname,
				   MAX_LEN - 1);
			  give_original_name (item_index
					      (current_item (filemenu)) + 1,
					      cwdinfo.head,
					      playopt->first->filename);
			  playopt->last = playopt->first;
			}
		      else
			{
			  playopt->last->next =
			    (PLAYLIST *) malloc (sizeof (PLAYLIST));
			  playopt->last = playopt->last->next;
			  memcheck (playopt->last);
			  playopt->totalsongs++;
			  playopt->last->songnum = playopt->totalsongs;
			  playopt->last->next = NULL;
			  strncpy (playopt->last->dirname, dirname,
				   MAX_LEN - 1);
			  give_original_name (item_index
					      (current_item (filemenu)) + 1,
					      cwdinfo.head,
					      playopt->last->filename);
			}
		    }
		}
	      else		//audio cd

		{
		  if (playopt->totalsongs == 0)
		    {
		      playopt->first =
			(PLAYLIST *) malloc (sizeof (PLAYLIST));
		      memcheck (playopt->first);
		      playopt->totalsongs++;
		      playopt->first->songnum = playopt->totalsongs;
		      playopt->first->next = NULL;
		      strcpy (playopt->first->dirname, "audio cd");
		      strncpy (playopt->first->filename,
			       item_name (current_item (filemenu)), 5);
		      playopt->last = playopt->first;
		    }
		  else
		    {
		      playopt->last->next =
			(PLAYLIST *) malloc (sizeof (PLAYLIST));
		      playopt->last = playopt->last->next;
		      memcheck (playopt->last);
		      playopt->totalsongs++;
		      playopt->last->songnum = playopt->totalsongs;
		      playopt->last->next = NULL;
		      strcpy (playopt->last->dirname, "audio cd");
		      strncpy (playopt->last->filename,
			       item_name (current_item (filemenu)), 5);
		    }
		}
	      werase (selwin);
	      wmove (selwin, 1, 1);
	      wprintw (selwin, "%s", item_name (current_item (filemenu)));
	      wrefresh (selwin);
	      // item_opts_off(current_item(filemenu),O_ONEVALUE);  
	      item_opts_off (current_item (filemenu), O_SELECTABLE);
	      box (filewin, ACS_VLINE, ACS_HLINE);
	    }
	  while (menu_driver (filemenu, REQ_DOWN_ITEM) == E_OK);
	  post_menu (filemenu);
	  wnoutrefresh (subfilewin);
	  wnoutrefresh (filewin);
	  doupdate ();

	  break;

	case F5:
	case 'b':
	  if (*mode == MP3MODE)
	    {
	      erase ();
	      printw
		("This may take a long time ! Please wait while I scanning your directory...");
	      refresh ();
	      napms (2000);
	      endwin ();
	      tmp_file = lmp3_tempnam ();
	      playlist = fopen (tmp_file, "w");
	      big_select_all (dirname, playlist);
	      fclose (playlist);
	      erase ();
	      refresh ();
	      endwin ();
	      printf ("Done !, now adding to playlist\n");
	      playlist = fopen (tmp_file, "r");

	      while (1)
		{
		  if (fscanf (playlist, "%[^\n]", bsa_dir) != 1)
		    break;
		  fscanf (playlist, "%[\n]", temp);
		  fscanf (playlist, "%[^\n]", bsa_file);
		  printf ("adding %s\n", bsa_file);
		  fscanf (playlist, "%[\n]", temp);
		  if (playopt->totalsongs == 0)
		    {
		      playopt->first =
			(PLAYLIST *) malloc (sizeof (PLAYLIST));
		      memcheck (playopt->first);
		      playopt->totalsongs++;
		      playopt->first->songnum = playopt->totalsongs;
		      playopt->first->next = NULL;
		      strncpy (playopt->first->dirname, bsa_dir, MAX_LEN - 1);
		      strncpy (playopt->first->filename, bsa_file,
			       MAX_LEN - 1);
		      playopt->last = playopt->first;
		    }
		  else
		    {
		      playopt->last->next =
			(PLAYLIST *) malloc (sizeof (PLAYLIST));
		      playopt->last = playopt->last->next;
		      memcheck (playopt->last);
		      playopt->totalsongs++;
		      playopt->last->songnum = playopt->totalsongs;
		      playopt->last->next = NULL;
		      strncpy (playopt->last->dirname, bsa_dir, MAX_LEN - 1);
		      strncpy (playopt->last->filename, bsa_file,
			       MAX_LEN - 1);
		    }
		}
	      curs_set (0);
	      fclose (playlist);
	      remove (tmp_file);
	      key = F10;
	      *quit = no;
	    }
	  break;

	case F1:
	case 'p':
	  do
	    {
	      reload = no;
	      plmenu (&reload);
	    }
	  while ((reload) == yes);
	  key = F10;
	  *quit = NO;
	  break;



	case 'q':
	  disable_curses_and_exit (0);
	  break;

	default:
	  break;
	}


    }


  for (count = 0; count < totalfile; count++)
    {
      free (filesname[count]);
      filesname[count] = NULL;
      free_item (item[count]);
    }
  // free_item(item[count+1]);
  unpost_menu (filemenu);
  free_menu (filemenu);

}

void
give_modified_name (const int itemnum, LIST * a, char *const temp)
{
  if (a->item_num != itemnum)
    give_modified_name (itemnum, a->next, temp);
  else
    {
      strncpy (temp, a->modified, MAX_CHAR + 1);
//        printw("itemnum = %d, str = %s \n",itemnum,a->modified);
      //       refresh();
      //        getch();
    }
}

void
give_dirname (const int itemnum, LIST * a, char *const temp)
{
  if (a->item_num != itemnum)
    give_dirname (itemnum, a->next, temp);
  else
    {
      strncpy (temp, a->filename, MAX_LEN + 1);
//        printw("itemnum = %d, str = %s \n",itemnum,a->modified);
      //       refresh();
      //        getch();
    }
}

void
give_original_name (const int itemnum, LIST * a, char *const temp)
{
  if (a->item_num != itemnum)
    give_original_name (itemnum, a->next, temp);
  else
    {
      strncpy (temp, a->filename, MAX_LEN);
//        erase();
      //      printw("itemnum = %d, str = %s \n",itemnum,a->modified);
      //        refresh();
      //        getch();
    }
}
