/*****************************************************************************
* Lien Mp3 Player is an audio CD and mp3 player for GNU/Linux                *
* Copyright (C) 2007 Tan Chee Kien. See file COPYING for licence details     *
*                                                                            *
*    This program is free software; you can redistribute it and/or modify    *
*    it under the terms of the GNU General Public License as published by    *
*    the Free Software Foundation; either version 2 of the License, or       *
*    any later version.                                                      *
*                                                                            *
*    This program is distributed in the hope that it will be useful,         *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*    GNU General Public License for more details.                            *
*                                                                            *
*    You should have received a copy of the GNU General Public License       *
*    along with this program; if not, write to the Free Software             *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    *  
*    USA                                                                     *
*                                                                            *
*    You can contact the author by sending an email to :                     *
*       gogota321@yahoo.com                                                  *
*****************************************************************************/


#ifndef _FILEMENU_H
#define _FILEMENU_H
#define _GNU_SOURCE

#include "lmp3.h"
#include <ncurses.h>
#include <menu.h>
#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>
#include <fnmatch.h>

// #define TRACE printf("%u  %s\n",__LINE__,__FILE__)
#define ISDIR 1
#define ISFILE 0
#define YES 1
#define NO 0
//#define MAX_LEN 100 // max character for filename to store in RAM
#define MAX_CHAR 66		/* max character for filename to display on screen, 
				   if over this character, the screen will display nothing
				 */

/* for display() */
#define ACDMODE 1		//audio cd mode
#define MP3MODE 0

struct list
{				// linked list to store dir files information
  char filename[MAX_LEN];	// the original filename
  char modified[MAX_CHAR + 1];	// string modified from filename to display on menu 
  int dir_or_file;		// is it dir or file ?
  int item_num;			// item index...start from 1
  struct list *next;		// pointer to next structure
};

typedef struct list LIST;

struct dirinfo
{				// main dir info structure
  LIST *head;			// pointer to first structure
  LIST *tail;			// pointer to last structure
  int total_item;		// total items in this dir
};

typedef struct dirinfo DIRINFO;


// return total item that is not dir in this dir 
int check_total_file (void);

// return total dir(s) in this dir
int check_total_dir (void);

// read all file(s) name from current dir, sort it in nice order
// then input it into structure DIRINFO
void make_file_array (const int totalfile);

// read all dir(s) name from current dir, sort it in nice order
// then input it into structure DIRINFO
void make_dir_array (const int totaldir);

// for qsort - using strcmp 
int sortme (const void *first, const void *second);

// append *string & DirOrFile to the last item of DIRINFO 
void append_to_list (const char *string, const int DirOrFile);

// print the linked list , for debugging only.. 
void print_list (LIST * const a);

//copy original filename string to another modified string with MAX_CHAR char
void copy_to_modified (LIST * const a);

//recursively clear the memory in linked list
void clear_list (LIST * a);

//display the file menu, quit = YES will make the whole program quit
// mode is either mp3 mode or audio cd mode 
void display (const int totalfile, int *const quit, int *const mode);

// fill temp with the correct modified filename string
void give_modified_name (const int itemnum, LIST * a, char *const temp);

// fill temp with the correct dirname 
void give_dirname (const int itemnum, LIST * a, char *const temp);

void give_original_name (const int itemnum, LIST * a, char *const temp);


#endif
