/*****************************************************************************
* Lien Mp3 Player is an audio CD and mp3 player for GNU/Linux                *
* Copyright (C) 2007 Tan Chee Kien. See file COPYING for licence details     *
*                                                                            *
*    This program is free software; you can redistribute it and/or modify    *
*    it under the terms of the GNU General Public License as published by    *
*    the Free Software Foundation; either version 2 of the License, or       *
*    any later version.                                                      *
*                                                                            *
*    This program is distributed in the hope that it will be useful,         *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*    GNU General Public License for more details.                            *
*                                                                            *
*    You should have received a copy of the GNU General Public License       *
*    along with this program; if not, write to the Free Software             *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    *  
*    USA                                                                     *
*                                                                            *
*    You can contact the author by sending an email to :                     *
*       gogota321@yahoo.com                                                  *
*****************************************************************************/

/* main function */
#define _GNU_SOURCE
#include "lmp3.h"
#include "audiocd.h"
#include "plfunc.h"
#include <ncurses.h>
#include <signal.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>
#include <unistd.h>
#include <dirent.h>
#include <fnmatch.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

//#define TRACE mvprintw(1,1,"%u  %s\n",__LINE__,__FILE__); refresh()

void givehelp (void);

LMP3GEN *playopt;
COM_OPT cmnd;

int
main (int argc, char **argv)
{
#define YES 1
#define NO 0

  int code;
  extern char *optarg;
  char conffile[50];
  FILE *stream;


  DIR *dir;
  char dirname[MAX_LEN];
  struct dirent *files;
  //int audio_cd_track;
  int fd;
  char *file;
  int _is_mp3 = yes;

  cmnd.randomize = no;
  cmnd.looping = no;
  cmnd.output_mode = -1;	// -1 meaning no option given

  cmnd.plfile = NULL;
  cmnd.mp3file = NULL;
  cmnd.match = no;
  cmnd.acd_track = 0;
  cmnd.cd_device = NULL;

  while ((code = getopt (argc, argv, "hrlo:p:m:c:")) != -1)
    {
      switch (code)
	{
	case 'r':
	  cmnd.randomize = yes;
	  continue;
	case 'l':
	  cmnd.looping = yes;
	  continue;
	case 'o':
	  if (strcmp (optarg, "S") == 0)
	    {
	      cmnd.output_mode = stereo;
	      continue;
	    }
	  else if (strcmp (optarg, "L") == 0)
	    {
	      cmnd.output_mode = left;
	      continue;
	    }
	  else if (strcmp (optarg, "R") == 0)
	    {
	      cmnd.output_mode = right;
	      continue;
	    }
	  else
	    {
	      givehelp ();
	      exit (-1);
	    }

	case 'p':
	  if (cmnd.mp3file != NULL)
	    {
	      printf ("You cannot use -m and -p together\n");
	      exit (-1);
	    }
	  else
	    {
	      if ((stream = fopen (optarg, "rt")) == NULL)
		{
		  printf ("cannot open %s, no such file\n", optarg);
		  exit (-1);
		}
	      fclose (stream);
	      cmnd.plfile = (char *) malloc (sizeof (char) * 50);
	      memcheck (cmnd.plfile);
	      strncpy (cmnd.plfile, optarg, 49);
	      continue;
	    }

	case 'm':
	  if (cmnd.plfile != NULL)
	    {
	      printf ("You cannot use -m and -p together\n");
	      exit (-1);
	    }
	  else
	    {
	      if ((stream = fopen (optarg, "rt")) != NULL)
		{
		  getcwd (dirname, sizeof (dirname));
		  dir = opendir (dirname);
		  for (;;)
		    {
		      if ((files = readdir (dir)) == NULL)
			break;
		      file = (files->d_name);
		      if (fnmatch (optarg, file, 0) == 0)
			{
			  strncpy (optarg, file, MAX_LEN);
			  cmnd.match = yes;
			  break;
			}
		    }
		  if (cmnd.match == no)
		    {
		      printf ("cannot open %s, no such file\n", optarg);
		      exit (-1);
		    }
		  fclose (stream);
		}
	      else
		_is_mp3 = no;
	      if (_is_mp3 == yes)
		{
		  cmnd.mp3file = (char *) malloc (sizeof (char) * MAX_LEN);
		  memcheck (cmnd.mp3file);
		  strncpy (cmnd.mp3file, optarg, MAX_LEN);
		}
	      else
		{
		  cmnd.acd_track = atoi (optarg);
		}
	      continue;
	    }

	case 'c':
	  if ((fd = open (optarg, O_RDONLY)) == -1)	//test whether device exist by open() it
	    {
	      printf ("Error, you didn't specify a correct cdrom device\n");
	      exit (-1);
	    }
	  close (fd);
	  cmnd.cd_device = (char *) malloc (sizeof (char) * 30);
	  strncpy (cmnd.cd_device, optarg, 30);
	  continue;
	case 'h':
	  givehelp ();
	  exit (0);
	default:
	  givehelp ();
	  exit (-1);
	}
    }
  //scanf("%[\n]");
  start_ncurses ();
  signal (SIGTERM, disable_curses_and_exit);	/*If you don't put this, the whole */
  signal (SIGQUIT, disable_curses_and_exit);	/*terminal will become nuts !      */
  signal (SIGINT, SIG_IGN);
  /*printw("line is %d\n",LINES);
     printw("cols is %d\n",COLS);
     refresh();
     getch(); */
  playopt = (LMP3GEN *) malloc (sizeof (LMP3GEN));
  init_play_option (playopt);
  init_conf_file (conffile);
  strncpy (playopt->conffile, conffile, 50);
  if ((stream = fopen (conffile, "r")) == NULL)	//configuration file not exist
    {
      erase ();
      printw ("Cannot create configuration file !\n");
      printw ("maybe your home directory is write protected ??\n");
      printw ("conf file is %s", playopt->conffile);
      refresh ();
      endwin ();
      disable_curses_and_exit (0);
    }
  else
    get_option_from_conffile (playopt, conffile);
  for (;;)
    {
      if ((cmnd.plfile == NULL) &&
	  (cmnd.mp3file == NULL) && (cmnd.acd_track == 0))
	{
	  filemenu ();
	  if (playopt->totalsongs == 0)	//the user didn't select any file !
	    continue;
	}
      else if ((cmnd.mp3file == NULL) && (cmnd.acd_track == 0))
	copy_file_to_playlist (playopt, cmnd.plfile);
      if ((cmnd.mp3file == NULL) && (cmnd.acd_track == 0))
	{
	  //print_playlist(playopt.first);
	  if ((playopt->randomize == yes) || (cmnd.randomize == yes))
	    makerdpl ();
	  bkgd (COLOR_PAIR (7));
	  if ((playopt->looping == YES) || (cmnd.looping == yes))
	    {
	      while (TRUE)
		playmp3 ();
	    }
	  else
	    playmp3 ();
	  if ((cmnd.plfile != NULL) || (cmnd.mp3file != NULL))	// using command line
	    break;
	}
      else			//user specify -m

	{
	  bkgd (COLOR_PAIR (7));
	  /*
	     printw("%d",atoi(cmnd.mp3file));
	     refresh();
	     getch();
	   */
	  if (_is_mp3 == yes)
	    playmp3_command ();
	  else
	    playacd_command ();
	  break;
	}
    }

  disable_curses_and_exit (0);
  return 0;
}

void
givehelp (void)
{
  printf ("Normally, just run lmp3 without any command options will do\n\n");
  printf ("The correct usage is :\n\n");
  printf ("-h : This help screen\n");
  printf ("-c : cdrom device (default is /dev/cdrom)\n");
  printf ("-r : enable random play\n");
  printf ("-l : enable looping\n");
  printf ("-oS : force stereo output (won't work for mono song)\n");
  printf ("-oL : force left channel output only\n");
  printf ("-oR : force right channel output only\n");
  printf ("-p <playlist filename> : use playlist\n");
  printf ("-m <Audio CD track no.> : play Audio CD track\n");
  printf
    ("-m <mp3file> : play this mp3 file, if the name is too hard to key in\
, you can use \'mp3 -m the*\' instead of \'lmp3 -m thedreamwithin.mp3\', lmp3 \
will only play the first song that match the pattern. You cannot play 2 \
songs or more with this option\n\n");
  printf ("example : lmp3 -rlp sting\n");
  printf
    ("This will play the playlist \'sting\' in random and looping mode\n");
  printf
    ("Lien Mp3 Player Version %s by Tan Chee Kien <gogota321@yahoo.com>\n",
     LMP3_VERSION);
}
