/*****************************************************************************
* Lien Mp3 Player is an audio CD and mp3 player for GNU/Linux                *
* Copyright (C) 2007 Tan Chee Kien. See file COPYING for licence details     *
*                                                                            *
*    This program is free software; you can redistribute it and/or modify    *
*    it under the terms of the GNU General Public License as published by    *
*    the Free Software Foundation; either version 2 of the License, or       *
*    any later version.                                                      *
*                                                                            *
*    This program is distributed in the hope that it will be useful,         *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*    GNU General Public License for more details.                            *
*                                                                            *
*    You should have received a copy of the GNU General Public License       *
*    along with this program; if not, write to the Free Software             *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    *  
*    USA                                                                     *
*                                                                            *
*    You can contact the author by sending an email to :                     *
*       gogota321@yahoo.com                                                  *
*****************************************************************************/

/* header file for almost all */
#ifndef _LMP3_H_
#define _LMP3_H_

#include <ncurses.h>

#define LMP3_VERSION "0.6.2b"

#define no 0
#define yes 1

#define MAX_LEN 300
#define cur_error(text) erase(); \
                        printw(#text); \
                        refresh(); \
                        endwin(); \
                        disable_curses_and_exit(0)


// code return by play(char *song)
// PREVIOUS means reverse to previous song
// RESET means return to song beginning
#define CONTINUE 1
#define PREVIOUS 0
#define RESET -1

#define LEFT_KEY 260
#define RIGHT_KEY 261

// for temporary file name
//define RECUR_PREFIX "l_rc"

typedef enum
{ stereo, left, right }
OUTPUT_TYPE;

struct playlist
{
  char dirname[MAX_LEN];
  char filename[MAX_LEN];
  int songnum;
  struct playlist *next;
};

typedef struct playlist PLAYLIST;

struct lmp3_gen
{
  int randomize;
  int looping;
  int totalsongs;
  // QUALITY_TYPE quality;
  OUTPUT_TYPE output_mode;
  PLAYLIST *first;
  PLAYLIST *last;
  char conffile[50];
  char *plfile;
  char *tempfile1;
  char *tempfile2;
  char cd_device[30];
};				//structure to hold many program options and information

typedef struct lmp3_gen LMP3GEN;

struct com_opt
{
  int randomize;
  int looping;
  OUTPUT_TYPE output_mode;
  char *plfile;
  char *mp3file;
  int match;			// 1 = found a mp3 file match with -m 
  int acd_track;
  char *cd_device;
};

typedef struct com_opt COM_OPT;	//structure holding command line options


/* turn off ncurses mode and exit the program */
void disable_curses_and_exit (int k);

/* this function accept a string pointed by original and fix it with \ 
 * for character that mean something to the shell so that the mpg123 
 * parameter will work correctly.  */
void fixstr (char *const original);

void playmp3 (void);


/* start ncurses */
void start_ncurses (void);

/* a function that check whether malloc is successful or not */
void memcheck (const void *ptr);

/* get total files from playlist */
int gettotalfile (const char *playlist);

/* return pointer that contains address of string that contains home
 * directory */
char *gethome (void);

/* option menu */
void optmenu (void);

/* playlist menu */
void plmenu (int *reload);

// filemenu
void filemenu (void);

//initialize the global structure LMP3GEN
void init_play_option (LMP3GEN * const a);

//check whether conf file exist or not, if not, ,will create one. Also will
//modify string conffile to contain correct $HOME/.mp3rc
void init_conf_file (char *const conffile);

// make the playlist random
void makerdpl (void);

//return a rand number from 0 to range-1
int giverand (int range);

// get option from configfile and input into a
void get_option_from_conffile (LMP3GEN * a, char *configfile);

//decoding the file 'song' see above for return int
int play (char *song);

void blank_lmp3gen (LMP3GEN * a);

//same as playmp3 except user using -m
void playmp3_command (void);

//delete all temporary file that start with prefix
void delete_temp_file (void);

//create a direcory call lmp3_tmp in user directory and return a temporary filename
char *lmp3_tempnam (void);


//select all files in start_path and all it's sub directory and write it to playlist
void big_select_all (char *start_path, FILE * playlist);

#endif
