/*****************************************************************************
* Lien Mp3 Player is an audio CD and mp3 player for GNU/Linux                *
* Copyright (C) 2007 Tan Chee Kien. See file COPYING for licence details     *
*                                                                            *
*    This program is free software; you can redistribute it and/or modify    *
*    it under the terms of the GNU General Public License as published by    *
*    the Free Software Foundation; either version 2 of the License, or       *
*    any later version.                                                      *
*                                                                            *
*    This program is distributed in the hope that it will be useful,         *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*    GNU General Public License for more details.                            *
*                                                                            *
*    You should have received a copy of the GNU General Public License       *
*    along with this program; if not, write to the Free Software             *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    *  
*    USA                                                                     *
*                                                                            *
*    You can contact the author by sending an email to :                     *
*       gogota321@yahoo.com                                                  *
*****************************************************************************/

/* Built in function for Lien MP3 */

#include "lmp3.h"
#include "plfunc.h"
#include <ncurses.h>
#include <signal.h>
#include <stdlib.h>
#include <pwd.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <fnmatch.h>

void
disable_curses_and_exit (int k)
{
  extern LMP3GEN *playopt;

  //signal (k, SIG_IGN);
  if (playopt->totalsongs != 0)
    clear_playlist (playopt->first);
  if (playopt->plfile != NULL)
    {
      unlink (playopt->plfile);
      free (playopt->plfile);
    }
  if (playopt->tempfile1 != NULL)
    {
      unlink (playopt->tempfile1);
      free (playopt->tempfile1);
    }
  if (playopt->tempfile2 != NULL)
    {
      unlink (playopt->tempfile2);
      free (playopt->tempfile2);
    }
  delete_temp_file ();
  free (playopt);
  nocbreak ();
  echo ();
  endwin ();
  exit (EXIT_SUCCESS);
}



void
start_ncurses (void)
{
  initscr ();
  if (start_color () == ERR)
    {
      erase ();
      printw ("error !!!!!!!!!\n");
      printw ("Ncurses Failed to start in colour mode\n");
      printw ("If you are running in xterm, make\n");
      printw ("sure it is a colour xterm !\n");
      printw ("This screen will disappear in 3 seconds..");
      refresh ();
      sleep (3);
    }
  if (COLS < 80)
    {
      erase ();
      printw ("Your terminal columns must be 80 or above\n");
      refresh ();
      disable_curses_and_exit (0);
    }
  if (LINES < 24)
    {
      erase ();
      printw ("Your terminal lines must be 24 or above\n");
      refresh ();
      disable_curses_and_exit (0);
    }
  cbreak ();
  noecho ();
  nonl ();
  keypad (stdscr, TRUE);
  init_pair (1, COLOR_GREEN, COLOR_BLACK);
  init_pair (2, COLOR_BLUE, COLOR_BLACK);
  init_pair (3, COLOR_YELLOW, COLOR_BLACK);
  init_pair (4, COLOR_MAGENTA, COLOR_BLACK);
  init_pair (5, COLOR_RED, COLOR_BLACK);
  init_pair (6, COLOR_CYAN, COLOR_BLACK);
  init_pair (7, COLOR_WHITE, COLOR_BLACK);
  init_pair (8, COLOR_GREEN, COLOR_BLUE);
  init_pair (11, COLOR_WHITE, COLOR_BLUE);
  init_pair (12, COLOR_BLUE, COLOR_BLUE);
  init_pair (13, COLOR_YELLOW, COLOR_BLUE);
  init_pair (14, COLOR_MAGENTA, COLOR_BLUE);
  init_pair (15, COLOR_RED, COLOR_BLUE);
  init_pair (16, COLOR_CYAN, COLOR_BLUE);
  init_pair (17, COLOR_WHITE, COLOR_GREEN);
  init_pair (18, COLOR_BLUE, COLOR_GREEN);
  init_pair (19, COLOR_RED, COLOR_GREEN);
  init_pair (20, COLOR_WHITE, COLOR_RED);
  init_pair (21, COLOR_BLUE, COLOR_RED);
  init_pair (22, COLOR_WHITE, COLOR_MAGENTA);
  init_pair (23, COLOR_WHITE, COLOR_CYAN);
  init_pair (24, COLOR_YELLOW, COLOR_CYAN);
  init_pair (25, COLOR_MAGENTA, COLOR_YELLOW);
  init_pair (26, COLOR_RED, COLOR_CYAN);
  init_pair (27, COLOR_WHITE, COLOR_YELLOW);

  curs_set (0);			//set cursor invisible

}

void
memcheck (const void *ptr)
{
  if (ptr == NULL)
    {
      erase ();
      refresh ();
      printw ("Fatal error !! Not enough memory !!, go buy more RAM !");
      refresh ();
      endwin ();
      exit (EXIT_FAILURE);
    }
}

char *
gethome (void)
{
  struct passwd *pws;
  long userid;
  char *homedir;

  homedir = (char *) malloc (50 * sizeof (char));
  memcheck (homedir);
  userid = getuid ();
  pws = getpwuid (userid);
  homedir = pws->pw_dir;
  return homedir;
}

void
init_play_option (LMP3GEN * const a)
{
  extern COM_OPT cmnd;

  a->randomize = no;
  a->looping = no;
  a->totalsongs = 0;
  // a.quality=full;
  a->output_mode = stereo;
  a->first = NULL;
  a->last = NULL;
  //strncpy(a->conffile,conffilename,50);
  a->plfile = NULL;
  a->tempfile1 = NULL;
  a->tempfile2 = NULL;
  if (cmnd.cd_device == NULL)
    strcpy (a->cd_device, "/dev/cdrom");
  else
    strncpy (a->cd_device, cmnd.cd_device, 30);
}

void
init_conf_file (char *const conffile)
{
  FILE *stream;
  char *homedir;

  homedir = gethome ();
  strcpy (conffile, homedir);
  strcat (conffile, "/.lmp3rc");

  if ((stream = fopen (conffile, "r")) == NULL)	//configuration file not exist

    {
      stream = fopen (conffile, "w");
      // fprintf(stream,"0\n");
      fprintf (stream, "0\n");
      fprintf (stream, "0\n");
      fprintf (stream, "0\n");
    }
  fclose (stream);
}

void
get_option_from_conffile (LMP3GEN * a, char *configfile)
{
  FILE *stream;
  char temp[10];

  stream = fopen (configfile, "r");
  fscanf (stream, "%[^\n]", temp);
  a->output_mode = atoi (temp);
  fscanf (stream, "%[\n]", temp);
  fscanf (stream, "%[^\n]", temp);
  a->randomize = atoi (temp);
  fscanf (stream, "%[\n]", temp);
  fscanf (stream, "%[^\n]", temp);
  a->looping = atoi (temp);
  fclose (stream);
}

void
blank_lmp3gen (LMP3GEN * a)
{
  clear_playlist (a->first);
  a->totalsongs = 0;
  a->first = NULL;
  a->last = NULL;
}
